#pragma once
class element {
    public:
       element * parent;
       element * child;
       element * prev;
       element * next;
       BOOL initial;
       BOOL selected;
    public:
       element() { init(); }
       element(const CString & name) { init(); classname = name; selected = initial = FALSE; }
    public:
       CString file;
       int line;
       CString classname;
    protected:
       void init() { parent = child = prev = next = NULL; line = 0; }
};

typedef CArray<element*, element*> CElementArray;

class Elements {
    public:
       //****************
       // ~Elements
       //****************
       virtual ~Elements() { clear(); }

       //****************
       // clear
       //****************
       void clear() {
          for(INT_PTR i = 0; i < elements.GetSize(); i++)
             delete elements[i];
          elements.SetSize(0);
       }

       //****************
       // lookup
       //****************
       element * lookup(const CString & name) {
          for(INT_PTR i = 0; i < elements.GetSize(); i++)
             { /* scan array */
              if(elements[i]->classname == name)
                 { /* found it */
                  return elements[i];
                 } /* found it */
             } /* scan array */
          return NULL;
       }

       //****************
       // enter
       //****************
       element * enter(const CString & name) {
          element * e = lookup(name);
          if(e != NULL)
             { /* already there */
              ASSERT(FALSE);  // TODO: better error reporting
              return NULL; // indicates error
             } /* already there */
          e = new element(name);
          elements.Add(e);
          return e;
       }

       //****************
       // sort
       //****************
       static int compare(const void * p1, const void * p2) {
          element ** e1 = (element **)p1;
          element ** e2 = (element **)p2;
          if((*e1)->classname < (*e2)->classname)
             return -1;
          if((*e1)->classname > (*e2)->classname)
             return 1;
          return 0;
       }

       void sort() {
          qsort(elements.GetData(), elements.GetSize(), sizeof(element*), compare);
       }

       //****************
       // GetFirstPosition
       //****************
       POSITION GetFirstPosition() {
          if(elements.GetSize() == 0)
             return (POSITION)NULL;
          else
             return (POSITION)1;  // note 1 larger than actual index so NULL is valid!
       }

       //****************
       // GetNext
       //****************
       element * GetNext(POSITION & p) {
          INT_PTR i = ((INT_PTR)p) - 1;
          element * e = elements[i];
          i++;
          if(i >= elements.GetSize())
             p = (POSITION)NULL;
          else
             p = (POSITION)(i + 1);
          return e;
       }

       //****************
       // Dump
       //****************
       void Dump() {
          TRACE("=============================================%d=====================================\n", elements.GetSize());
          for(INT_PTR i = 0; i < elements.GetSize(); i++)
             { /* show each */
              TRACE("%p: \"%s\" %s%s [%s (%d)]\n    parent %p (%s) child %p (%s)\n    next %p (%s)  prev %p (%s)\n",
                    elements[i],
                    elements[i]->classname,
                    elements[i]->initial ? _T("+") : _T(""),
                    elements[i]->selected ? _T("*") : _T(""),
                    elements[i]->file,
                    elements[i]->line,
                    elements[i]->parent,
                    elements[i]->parent == NULL ? _T("NULL") : elements[i]->parent->classname,
                    elements[i]->child,
                    elements[i]->child == NULL ? _T("NULL") : elements[i]->child->classname,
                    elements[i]->next,
                    elements[i]->next == NULL ? _T("NULL") : elements[i]->next->classname,
                    elements[i]->prev,
                    elements[i]->prev == NULL ? _T("NULL") : elements[i]->prev->classname);
             } /* show each */
       }
protected:
   CElementArray elements;

};

       
